
/*
 *
 * ststuff.c - Retrofit routines
 *             for the Atari St's
 *
 */

#include <stdio.h>
#include <osbind.h>
#include "astat.h"

#define BSIZE 80;
#define ENVIRON "environ";

char raw=0;
#ifdef DEBUG
int stdebug=0;
#endif

/*
 * Get file statistics - sort of like stat(2)
 */

int getstat(fname, buf)
register char *fname;
register struct stat *buf;
{
	register struct stat *save_dta;
	register int status;

	/* Save old DTA */
	save_dta = (struct stat *)Fgetdta();

	/* set the new DTA */
	Fsetdta(buf);

	/* Find file stat */
	status = Fsfirst(fname, 0);

	/* reset dta */
	Fsetdta(save_dta);
	
	/* return status */
	return (status == 0) ? 1 : 0;
	
}


	
/*
 * time - stuff current time & date into long (ptr passed)
 *
 */
time(t)
long *t;
{

	*t = Gettime();	/* Ikbd's time */
}

/************************************************************************/
/*									*/
/*	MICRO C-Shell for the Atari ST					*/
/*	Written by David Beckemeyer					*/
/*									*/
/*	(C) Copyright 1985						*/
/*	David Beckemeyer						*/
/*	All Rights Reserved						*/
/*									*/
/*	Unauthorized Duplication Prohibited				*/
/*									*/
/************************************************************************/

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif

struct basepage {
	char *ltpa;
	char *htpa;
	char *lcode;
	long codelen;
	char *ldata;
	long datalen;
	char *lbss;
	long bsslen;
	char *dta;
	char *parent;
	long rsvd;
	char *env;
};

extern struct basepage *_basepag;


/* locate variable name in environment string */
static char *findenv(s)
char *s;
{
	char name[20];
	char *p;

	if (s) {
		strcpy(name, s);
		strcat(name, "=");
	}
	for (p = _basepag->env; *p; p++) {
		if (s) {
			if (strcmp(p, name) == 0)
				break;
		}
		while (*p)
			p++;
	}
	return(p);
}


	
char *searchenv(name)
char *name;
{
	char *p;

	p = findenv(name);
	if (*p) {
		while (*p)
			p++;
		return(++p);
	}
	return(NULL);
}

static FILE *fd;
/*sfile: look for a line in a file called environ that starts with
  name.
 */
char *sfile(name,result)
char *name, *result;
{
	char *p=NULL;
	int len;
	if ((fd=fopen("environ","r"))==NULL) return p;
	while (fgets(result,80,fd)!=NULL) {
		if ((p=strrchr(result,'\n'))!=NULL) *p='\0';
		len=stcpm(result,name,&p);
		if (len==strlen(name)) {
			p = &result[len];
			while (*p && ((*p!=' ') && (*p!='='))) p++;
			break;
		}
		p=NULL;
	}
	fclose(fd);
	return p;
/*end sfile*/}
		
/*getenv: look for name in the environment.  If not found, search a file
 called environ.
 */
char *getenv(name)
char *name;
{
	static char value[80];
	char *p=NULL;
	if ((p=searchenv(name))!=NULL) return p;
	return (sfile(name,value));
/*end getenv*/}

access(path,mode)
char *path;
int mode;
{
	struct stat stat;
	int result=-1;
	if (getstat(path,&stat)) 
		if ((stat.st_mode&mode)==mode) result=0;
	return result;
/*end access*/}

/*keys_waiting: return the number of keys queued, but not read*/
keys_waiting()
{
	struct iorec {
		long ibuf;
		short ibufsize;
		short ibufhd;
		short ibuftl;
		short ibuflow;
		short ibufhi;
	} *p;
	int len;
	p=(struct iorec *) Iorec(0);
	len=p->ibufhd-p->ibuftl;
	if (len<0) len+=p->ibufsize;
	return len;
/*end keys_waiting*/}

static char *mdays[7]={
	"Tue",
	"Wed",
	"Thu",
	"Fri",
	"Sat",
	"Sun",
	"Mon"};
static char *mons[12]={
	"Jan",
	"Feb",
	"Mar",
	"Apr",
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Oct",
	"Nov",
	"Dec"};
static int dm[13]={0,0,31,59,90,120,151,181,212,243,273,304,334};
/*ctime: convert a long integer retrieved from the timer to a date
 and time string.
 */
char *ctime(clock)
long *clock;
{
	static char s[26];
	short sec, mins, hour, mday, mon, year, wday;
	int days;
	sec=(*clock)&0x1f;
	mins=((*clock)&0x000007e0)>>5;
	hour=((*clock)&0x0000f100)>>11;
	mday=((*clock)&0x001f0000)>>16;
	mon= ((*clock)&0x01e00000)>>21;
	year=((*clock)&0xfe000000)>>25;
	days=(year*365)+(year/4)+1+dm[mon]+mday;
	wday=days%7;
	sprintf(s,"%s %s %2d %2d:%2d:%2d %4d\n", mdays[wday], mons[mon-1],
		mday, hour, mins, sec, year+1980);
	return &s[0];
/*end ctime*/}
	
/*sleep: do nothing for a little while*/
sleep(delay)
{
	int i;
	while (delay--) for (i=0; i<10000; i++) ;
/*end sleep*/}

/*readch: get a character from the keyboard and translate, if necessary*/
char readch()
{
	char c, hi, lo;
	long keycode=(raw) ? Crawcin() : Cconin();
	lo=keycode&0xff;
	hi=(keycode>>16)&0xff;
	switch (hi) {
		case 0x6d : {c=(lo==0x11) ? 'B' : 'b'; break;}
		case 0x6e : {c=(lo==0x00) ? 'J' : 'j'; break;}
		case 0x6f : {c=(lo==0x13) ? 'N' : 'n'; break;}
		case 0x6a : {c=(lo==0x14) ? 'H' : 'h'; break;}
		case 0x6b : {c='.'; break;}
		case 0x6c : {c=(lo==0x1e) ? 'L' : 'l'; break;}
		case 0x67 : {c=(lo==0x17) ? 'Y' : 'y'; break;}
		case 0x68 : {c=(lo==0x18) ? 'K' : 'k'; break;}
		case 0x69 : {c=(lo==0x19) ? 'U' : 'u'; break;}
		case 0x62 : {c='?'; break;}
		case 0x48 : {c=(lo==0x00) ? 'k' : 'K'; break;}
		case 0x4b : {c=(lo==0x00) ? 'h' : 'H'; break;}
		case 0x4d : {c=(lo==0x00) ? 'l' : 'L'; break;}
		case 0x50 : {c=(lo==0x00) ? 'j' : 'J'; break;}
		default : {c= (lo=='\r') ? '\n' : lo;}
	}
	return (c);
/*end readch*/}

/*st_exit: ask for a return before exiting*/
st_exit(code)
int code;
{
	cputs("\nTouch Return");
	while (readch()!='\n') ;
	exit(code);
/*end st_exit*/}

#ifdef DEBUG
print(str)
char *str;
{
	while (*str) Cprnout(*str++);
/*end print*/}
#endif
